#!/bin/bash
#
#       Copyright (C) MOXA Inc. All rights reserved.
#       Copyright (C) 2015       Andy CY Chen <AndyCY.Chen@moxa.com>
#
#       This software is distributed under the terms of the
#       MOXA License.  See the file COPYING-MOXA for details.
#
#       Smart Recovery Fucntion
#               backup, recovery automatically. This version support DA-682A seris
#

BUILD_DATE="15120901"
#LOCAL_DISK="sda"
#TARGET_DISK="sdb"
#TARGET="sdb1"
MODEL=`kversion | awk '{print $1}'`;
VERSION=`uname -r`
source /etc/smart_recovery/.srcfg

if ! [ -d "/tmp/p1" ] ; then
	mkdir -p /tmp/p1
fi

if ! [ -d "/tmp/p2" ] ; then
	mkdir -p /tmp/p2
fi


# $1 indicates set to 0 = master (F1) or 1 = slvae (F3) 
# $2 is storage device, ex: /dev/sdb 
# set_port() will set port to cmos according to 
# the device types (USB, CF, mSATA, SATA)
set_port()
{


local target_port_value=0

# set master #
if [ 0 == $1 ]; then
	target_addr="F1"		
# set slave #
else
	target_addr="F3"
fi

local target_device=$2

# get device type #
local type=`udevadm info -q path -n $target_device | awk -F "/" '{printf $5}'`

case $type in
# USB #
	"usb2")
		target_port_value=`udevadm info -q path -n $target_device | awk -F "/" '{printf $7}' | awk -F "." '{printf $2}'`
		# the value should -1 and shift 4 bits to be port value #		
		target_port_value=$(($target_port_value - 1 ))
		target_port_value=$(($target_port_value << 4 ))
		;;
# CF/mSATA/SATA #
	*)
		# check whether being sda #
		if [ "x/dev/sda" == x$target_device ]; then
			if [ $MODEL == "DA-682A" ]; then
				target_port_value=16
			else	# DA-681A # 
				target_port_value=0
			fi
		else
			local tmp_port_value=`udevadm info -q path -n $target_device | awk -F "/" '{printf $7}' | awk -F ":" '{printf $3}'`

			if [ "x0" == x$tmp_port_value ]; then
				if [ $MODEL == "DA-682A" ]; then
					target_port_value=32
				else
					target_port_value=16
				fi
			else
				if [ $MODEL == "DA-682A" ]; then
					target_port_value=48
				else
					target_port_value=32
				fi
			fi			
		fi
		;;
esac

# write to cmos #
/sbin/sr_wr_cmos 1 $target_addr $target_port_value

}

#exit 0
#
#  umount all target partition
#
#  e.g. umount_target sda1
#
umount_target ()
{
	local target="$1"

	while [ x"$(mount | grep $target)" != x"" ]
	do
		umount $1
		sleep 1
	done

}

#
#  make a list of all partitions
#  return array of all partitions
#
#  e.g. list_all_partition sda
#
list_all_partition ()
{
	local target="$1"
	echo "$(fdisk -l /dev/${target} | awk '{print $1}' | grep /dev/${target})"
}


#
#  1. Check target disk exist.
#  2. Check target disk MBR is as same as local disk.
#  3. Check target disk partition can be mount.
#
#check_storage ()
#{
#	local fail=0
	
	#  TODO

#}

#
#  restore MBR
#  	$1: local disk
#  	$2: target disk
#
#  e.g. dd_MBR $LOCAL $TARGET
#
dd_MBR ()
{

	local local_disk="$1"
	local target_disk="$2"
	
	dd if=$local_disk of=$target_disk bs=512k count=1	

}

#  
#  fdisk resize last partition
#  	$1: local disk
#  
#  e.g. fdisk_resize sda
#  
fdisk_resize ()
{

        local local_disk="$1"
        local local_all_par=($(list_all_partition ${local_disk}))
        local local_last_par="${local_all_par[${#local_all_par[@]}-1]}"
        local local_last_par_num=${local_last_par:$(expr ${#local_last_par}-1):1}
	local num_1=1
	
	umount_target "$local_disk$num_1"

        if [ x"1" == x"${#local_all_par[@]}" ]; then
                fdisk ${local_disk} << EOF
d
n
p



w
EOF

        else
                fdisk ${local_disk} << EOF
d
${local_last_par_num}
n
p
${local_last_par_num}


w
EOF

        fi

}

mkfs_parts()
{

        local local_disk=$1

        # Get number of partition of $localdisk #
        local_all_par=`fdisk -l /dev/sdb | awk '{print $1}' | grep /dev/sdb | wc -l`

        #echo $local_all_par

        for i in `seq 1 1 $local_all_par`
        do
                # mkfs the partition #
                mkfs -t ext4 "$local_disk$i"
        done
}



#
#  rsync all data from local to target partition
#  
#
#  e.g. rsync_target local/ target/
#
rsync_target ()
{
	num_1="1"
	local local_par="$1$num_1"
	local target_par="$2$num_1"

	# umount to avoid busy #
	#umount_target $local_par
	umount_target $target_par

	if [ x"" != x`df | grep /tmp/p1` ]; then	
		umount -f /tmp/p1	
	fi
	
	if [ x"" != x`df | grep /tmp/p2` ]; then
		umount -f /tmp/p2
	fi

	# mount partitions to /tmp/p1 and p2 respecttively #
	mount $local_par /tmp/p1
	mount $target_par /tmp/p2

	# -a, archive mode
        # -v, increase verbosity
        # -P, keep partially transferred files
	rsync -aquP --delete /tmp/p1/* /tmp/p2 

	# umount after finish #
	umount -f /tmp/p1
	umount -f /tmp/p2

	# record the last backup time #
	source /etc/smart_recovery/.srcfg
	echo "MASTER_STORAGE=$MASTER_STORAGE" > /etc/smart_recovery/.srcfg
	echo "SLAVE_STORAGE=$SLAVE_STORAGE" >> /etc/smart_recovery/.srcfg
	echo "AUTO_SYNC=$AUTO_SYNC" >> /etc/smart_recovery/.srcfg
	echo "LAST_SYNC_DATE=\"`date`\"" >> /etc/smart_recovery/.srcfg	

	return 0

}

#fdisk_resize $LOCAL_DISK
#fdisk_resize mmcblk0


#check_storage


#umount_target $TARGET
#echo done



#str=($(list_all_partition $TARGET_DISK))
#
#echo ${str[0]}
#echo ${str[1]}
#ERROR CODE 

#****************** run_check_storage ****************
# error code:
#
#	1=storage not found		
#	2=partition is not same
#	3=filesystem type is not same
#	4=storage is not mount
#	
#	5=master storager is not mount
#	6=Backup hard disk size is too small
#
#******************************************************

run_check_storage ()
{
	local local_storage="$1";
	local target_storage="$2";
	local local_storage_part_num="";
	local target_storage_part_num="";
	local local_storage_type="";
	local target_storage_type="";
	
	#Confirm the existence of storage device
	if ! [ -e "$local_storage" ] || ! [ -e "$target_storage" ]; then
		return 1;
	fi

	# get each number of partition #
	local_storage_part_num=`ls -la $local_storage* | wc -l`
	target_storage_part_num=`ls -la $target_storage* | wc -l`
	local_storage_part_num=$((local_storage_part_num-1))
	target_storage_part_num=$((target_storage_part_num-1))


	# check number of  partition
	if [ "$local_storage_part_num" != "$target_storage_part_num" ] ; then
			return 2;
	fi

	#check filesystem type && check storage mount
	#local_storage_part_num=$((local_storage_part_num - 1)) 

	for p in `seq 1 $local_storage_part_num`
	do


		# check the partition type #
		local_storage_type=`blkid -o value -s TYPE $local_storage$p`
		target_storage_type=`blkid -o value -s TYPE $target_storage$p`	
		if [ x"$local_storage_type" != x"$target_storage_type" ] ; then
			return 3;
		fi
		
		# test mount partition of master #
		mount $local_storage$p /tmp/p1
		if [ "$?" -ne "0" ] ; then
			return 5;
		fi
		umount -f /tmp/p1
	
		# test mount partition of slave #
		mount $target_storage$p /tmp/p1
		if [ "$?" -ne "0" ] ; then
			return 5;
		fi
		umount -f /tmp/p1
		
	done

	#check storage size, if local_stroage used size > target_stroage total size is failed
	local local_storage_total_size=`fdisk -s $local_storage`
	local target_storage_total_size=`fdisk -s $target_storage` 
	
	if [ x"$local_storage_size" != x"$target_storage_size" ] ; then
		
		local local_storage_used_size="df -P | grep "$local_storage"1 | awk -F " " '{print $3}'"	
		
		if [ "$local_storage_used_size" -gt "$target_storage_total_size" ] ; then
			return 6;
		fi
	fi

	echo 0;

}

#run_check_storage $MASTER_STORAGE $SLAVE_STORAGE

MODEL=`kversion | awk '{print $1}'`;
VERSION=`uname -r`

# $1 = TYPE , local storage -> 0 = master, 1 = backup

run_check_label()
{
	local local_storage="$1";
	local target_storage="$2";
	local target_label="$MODEL";
	local initrd="";
	local vmlinuz="";
	local type="$1"
	local vmlinux_path="/boot/vmlinuz-$VERSION"
	local NUM=""
	local tmp=""


	#new local stroage
	mount "$local_storage" /tmp/p1/

	if [ "$?" -ne "0" ] ; then
		return 1;
	fi

	if [ -f "/tmp/p1/boot/grub/grub.cfg" ] ; then
		chmod 777 /tmp/p1/boot/grub/grub.cfg;
	else
		return 2;
	fi

	if ! [ -f "/tmp/p1/etc/fstab" ] ; then
		return 3;
	fi


	# Get label #
	local local_label=`blkid -o value -s LABEL $local_storage`

	if [ x"$local_label" != x"$MODEL" ] && [ x"$local_label" != x"$MODEL-BAK" ] ; then

		# change the local label to $MODEL #
		local_label="$MODEL"
		e2label "$local_storage" "$local_label"	
		if [ "$?" -ne "0" ] ; then
			return -4;
		fi

	
		# replace the label in grub.cfg #
		old_label=`cat /tmp/p1/boot/grub/grub.cfg | grep "$vmlinux_path" | sed -n '1p' | awk -F " " '{print $3}'`
		new_label="root=LABEL=$target_label"
		old_label=$(echo $old_label | sed 's|/|\\/|g')
		new_label=$(echo $new_label | sed 's|/|\\/|g')
		sed -i "s/$old_label/$new_label/g" /tmp/p1/boot/grub/grub.cfg

		# replace the label in fstab #
		NUM=`cat /tmp/p1/etc/fstab | grep -E "(UUID|LABEL)" | wc -l`	
		for ((i = 1; i <= NUM; i=i+1));do

        		tmp=`cat /tmp/p1/etc/fstab | grep -E "(UUID|LABEL)" | sed -n "$i"p | awk -F " " '{print $2}'`

        		if [ "$tmp" = "/" ] ; then
				old_fstab_label=`cat /tmp/p1/etc/fstab | grep -E "(UUID|LABEL)" | sed -n "$i"p | awk '{print $1}'`
                		break;
        		fi
		done
		old_fstab_label=$(echo $old_fstab_label | sed 's|/|\\/|g')
		sed -i "s/$old_fstab_label/LABEL=$local_label/g" /tmp/p1/etc/fstab


	fi

	sync;

	umount /tmp/p1/

	if [ "$?" -ne "0" ] ; then
		return -5;
	fi

	#rm -rf /tmp/p1

	#check backup partition label	
	if [ x"$MODEL" == x"$local_label" ] ; then
		target_label="$MODEL-BAK";
	fi
	
	e2label "$target_storage" $target_label
	
	if [ "$?" -ne "0" ] ; then
		return -6;
	fi
	
	mount "$target_storage" /tmp/p2/

	if [ "$?" -ne "0" ] ; then
		return -7;
	fi

	#ctrl+v  + ctrl old

	old_label=`cat /tmp/p2/boot/grub/grub.cfg | grep "$vmlinux_path" | sed -n '1p' | awk -F " " '{print $3}'`
	
	new_label="root=LABEL=$target_label"
	old_label=$(echo $old_label | sed 's|/|\\/|g')
	new_label=$(echo $new_label | sed 's|/|\\/|g')
	
	sed -i "s/$old_label/$new_label/g" /tmp/p2/boot/grub/grub.cfg

	old_uuid_line="search --no-floppy --fs-uuid --set=root"
	slave_uuid=`blkid -o value -s UUID $target_storage`
	new_uuid_line="search --no-floppy --fs-uuid --set=root $slave_uuid"

	# update slave's UUID #
	sed -i "s/.*$old_uuid_line.*/$new_uuid_line/g" /tmp/p2/boot/grub/grub.cfg	


	NUM=`cat /tmp/p2/etc/fstab | grep -E "(UUID|LABEL)" | wc -l`	

	for ((i = 1; i <= NUM; i=i+1));do

        	tmp=`cat /tmp/p2/etc/fstab | grep -E "(UUID|LABEL)" | sed -n "$i"p | awk -F " " '{print $2}'`

        	if [ "$tmp" = "/" ] ; then
			old_fstab_label=`cat /tmp/p2/etc/fstab | grep -E "(UUID|LABEL)" | sed -n "$i"p | awk '{print $1}'`
                	break;
        	fi
	done

	old_fstab_label=$(echo $old_fstab_label | sed 's|/|\\/|g')
	sed -i "s/$old_fstab_label/LABEL=$target_label/g" /tmp/p2/etc/fstab

	sync;
	umount -f /tmp/p2

	if [ "$?" -ne "0" ] ; then
		return -8;
	fi

	return 0;
}

# clone disk for backup #
clone_storage_config() {
        dd_MBR $MASTER_STORAGE $SLAVE_STORAGE
        fdisk_resize $SLAVE_STORAGE
	mkfs_parts $SLAVE_STORAGE
}

# clone disk for recovery #
clone_storage_recovery() {
	dd_MBR $SLAVE_STORAGE $MASTER_STORAGE
	fdisk_resize $MASTER_STORAGE
	mkfs_parts $MASTER_STORAGE
}


	num_1=1
	#  run_check_storage $MASTER_STORAGE $SLAVE_STORAGE
	# run_check_label $MASTER_STORAGE$num_1 $SLAVE_STORAGE$num_1

	# these function is tested
	# dd_MBR $MASTER_STORAGE $SLAVE_STORAGE
	# fdisk_resize $SLAVE_STORAGE
	# mkfs_parts $SLAVE_STORAGE
	# rsync_target $MASTER_STORAGE $SLAVE_STORAGE	
