#!/bin/bash
#
#       Copyright (C) MOXA Inc. All rights reserved.
#       Copyright (C) 2015       Andy CY Chen <AndyCY.Chen@moxa.com>
#
#       This software is distributed under the terms of the
#       MOXA License.  See the file COPYING-MOXA for details.
#
#       Smart Recovery rc
#               backup, recovery automatically. This version support DA-682A seris
#

# read config #
source /etc/smart_recovery/.srcfg

perror() {
	echo "Smart Recovery: $1" >> /etc/smart_recovery/sr.log
}

call_backup() {

        # read config #
        #sync immediately
        source /sbin/smart_recovery_func.sh

	local num_1=1
	local master_storage="$MASTER_STORAGE$num_1"
	local slave_storage="$SLAVE_STORAGE$num_1"

	perror "[Backup] Starting backup from $MASTER_STORAGE to $SLAVE_STORAGE"

        # check storage #
        ret=`run_check_storage $MASTER_STORAGE $SLAVE_STORAGE`
        ret=$?
	if [ "x0" != x$ret ]; then
       		# try to repair the repairable error (error code: 2, 3, 5) #
        	if [ 2 == $ret ] || [ 3 == $ret ] || [ 5 == $ret ]; then
        		# try to repair with DD_MDR, FDISK and MKFS
                	# call_repair
                	clone_storage_config

                	# check storage again #
                	ret2=`run_check_storage $MASTER_STORAGE $SLAVE_STORAGE`
			ret2=$?
			if [ "x0" != x$ret2 ]; then # the error repair failed #
				perror "[Backup] Storage error code2: $ret2"
				return 0	
			fi
        	else # the error can not be repaired #
			perror "[Backup] Storage error code1: $ret"
			return 0
		fi
        fi

	# check the label of slave, it should be like "DA-682A-BAK" #
	slave_label="`blkid -o value -s LABEL $slave_storage`"	
	if [ x"$MODEL-BAK" != x$slave_label ]; then
		perror "[Backup] The label of slave is wrong, it might be pluging wrong storage on selected slave storage position."
		return 0
	fi

	# everything ok -> try to sync master and slave  #
	rsync_target $MASTER_STORAGE $SLAVE_STORAGE

	# print the rsync result #
	if [ $? == 0 ]; then
		perror "[Backup] Backup from $MASTER_STORAGE to $SLAVE_STORAGE succuess"
	else
		perror "[Backup] Rsync failed"
	fi
}

call_recovery() {


	perror "[Recovery] Try to recovery from slave to master !! "

        # read config #
        #sync immediately
        source /sbin/smart_recovery_func.sh

        local num_1=1
        local master_par="$MASTER_STORAGE$num_1"
        local slave_par="$SLAVE_STORAGE$num_1"

	perror "[Recovery] Starting recovery from $SLAVE_STORAGE to $MASTER_STORAGE"
#        perror "Starting backup from $MASTER_STORAGE to $SLAVE_STORAGE"

        # check storage #
        ret=`run_check_storage $MASTER_STORAGE $SLAVE_STORAGE`
        ret=$?
        if [ "x0" != x$ret ]; then
                # try to repair the repairable error (error code: 2, 3, 5) #
                if [ 2 == $ret ] || [ 3 == $ret ] || [ 5 == $ret ]; then
                        # try to repair with DD_MDR, FDISK and MKFS
                        # call_repair
                        #clone_storage_config
			clone_storage_recovery			

                        # check storage again #
                        ret2=`run_check_storage $MASTER_STORAGE $SLAVE_STORAGE`
                        ret2=$?
                        if [ "x0" != x$ret2 ]; then # the error repair failed #
                                perror "[Recovery] Storage error code2: $ret2"
                                return 0
                        fi
                else # the error can not be repaired #
                        perror "[Recovery] Storage error code1: $ret"
                        return 0
                fi
        fi

        # check the label of slave, it should be like "DA-682A-BAK" #
        #slave_label="`blkid -o value -s LABEL $slave_storage`"
	#master_label="`blkid -o value -s LABEL $master_storage`"
        #if [ x"$MODEL" != x$master_label ]; then
        #        perror "The label of master is wrong, it might be pluging wrong storage on selected slave storage position."
        #        return 0
        #fi

        # everything ok -> try to sync slave to  master #
        #rsync_target $MASTER_STORAGE $SLAVE_STORAGE # it is backup direct
	rsync_target $SLAVE_STORAGE $MASTER_STORAGE 

        # print the rsync result #
        if [ $? == 0 ]; then
                #perror "Backup from $MASTER_STORAGE to $SLAVE_STORAGE succuess"
		perror "[Recovery] Rsync from $SLAVE_STORAGE to $MASTER_STORAGE success"
        else
                perror "[Recovery] Rsync failed"
        fi

	# broken master label -> force to relabel #
	e2label $master_par "12345"	

	# run check label to set master label (in /boot/grub/grub.cfg and /etc/fstab) #	
	run_check_label $master_par $slave_par

	if [ $? == 0 ]; then
		perror "[Recovery] Set label success"
	else
		perror "[Recovery] Set label failed"
	fi

	# update master uuid #
	mount $master_par /tmp/p1
        old_uuid_line="search --no-floppy --fs-uuid --set=root"
        master_uuid=`blkid -o value -s UUID $master_par`
        new_uuid_line="search --no-floppy --fs-uuid --set=root $master_uuid"
        sed -i "s/.*$old_uuid_line.*/$new_uuid_line/g" /tmp/p1/boot/grub/grub.cfg
	umount /tmp/p1

	if [ $? == 0 ]; then
		perror "[Recovery] Recovery from slave to master is success."
		reboot
	else
		perror "[Recovery] Recovery from slave to master is failed."
	fi
}

# check disk label, if boot from slave, try to recovery #

# get model 
model=`kversion | awk '{print $1}'`
# get label 
boot_label=`cat /boot/grub/grub.cfg | grep LABEL | awk -F "=" '{printf $3}' | awk -F " " '{printf $1}' `

# clear watchdog timer #
ret=`/sbin/sr_wr_cmos 3 B2 120`
echo "clear watchdog timer `date`" >> /etc/smart_recovery/sr.log

# label should be like "DA-682A or DA-682A-BAK #
if [ x$boot_label == x$model ]; then
	# boot from master, check whether automatically sync #
	if [ "xON" == "x"$AUTO_SYNC ]; then
		call_backup
		echo "boot from master and autosync is on" `date` >> /etc/smart_recovery/sr.log
	else
		perror "AUTO_SYNC: $AUTO_SYNC"	
		echo "AUTO_SYNC: $AUTO_SYNC" `date` >> /etc/smart_recovery/sr.log
	fi
	exit 0
fi

if [ x$boot_label == x$model"-BAK" ]; then
	# boot from slave -> recovery  # 
	echo "boot from slave !" `date` >> /etc/smart_recovery/sr.log
	
	# check boot from recovery path (i.e.,CMOS 0xF5 = 1) #
	recovery_path=`/sbin/sr_wr_cmos 0 F5 | awk -F "=" '{printf $2}'`

        # get MOXA_SR_CMOS_BOOT_FAIL_COUNT #
        boot_fail_count=`/sbin/sr_wr_cmos 0 EE`

	# if recovery path == 0, it should be boot manually -> exit #
	if [ "x0" == x$recovery_path ]; then
		# write E6 #
		/sbin/sr_wr_cmos 1 E6 $boot_fail_count
		exit 0;
	fi
	
	boot_fail_count=$(($boot_fail_count - 1))
	# write E6 #	
	/sbin/sr_wr_cmos 1 E6 $boot_fail_count	

	# else -> recovery master from slave #
	call_recovery
fi
